from __future__ import unicode_literals

import os
from glob import glob

_this_dir = os.path.dirname(os.path.abspath(__file__))
_opencc_rootdir = os.path.abspath(os.path.join(_this_dir, '..', '..'))
_test_assets_dir = os.path.join(_opencc_rootdir, 'test', 'testcases')


def test_import():
    import opencc  # noqa


def test_init_delete_converter():
    import opencc

    for config in opencc.CONFIGS:
        converter = opencc.OpenCC(config)
        del converter


def test_conversion():
    import opencc

    for inpath in glob(os.path.join(_test_assets_dir, '*.in')):
        pref = os.path.splitext(inpath)[0]
        config = os.path.basename(pref)
        converter = opencc.OpenCC(config)
        anspath = '{}.{}'.format(pref, 'ans')
        assert os.path.isfile(anspath)

        with open(inpath, 'rb') as f:
            intexts = [l.strip().decode('utf-8') for l in f]
        with open(anspath, 'rb') as f:
            anstexts = [l.strip().decode('utf-8') for l in f]
        assert len(intexts) == len(anstexts)

        for text, ans in zip(intexts, anstexts):
            assert converter.convert(text) == ans, \
                'Failed to convert {} for {} -> {}'.format(pref, text, ans)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import glob
import sys

from common import sort_items

if len(sys.argv) < 2:
    print("Sort the dictionary")
    print(("Usage: ", sys.argv[0], "[directory]"))
    exit(1)

directory = sys.argv[1]
files = glob.glob(directory + "/*")
for filename in files:
    print(filename)
    sort_items(filename, filename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys

from common import sort_items

if len(sys.argv) < 2:
    print("Sort the dictionary")
    print(("Usage: ", sys.argv[0], "[input] ([output])"))
    exit(1)

input = sys.argv[1]

if len(sys.argv) < 3:
    output = input
else:
    output = sys.argv[2]

sort_items(input, output)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from common import find_target_items

if len(sys.argv) != 3:
    print("Find the value keyword in all pairs")
    print(("Usage: ", sys.argv[0], "[input] [keyword]"))
    exit(1)

find_target_items(sys.argv[1], sys.argv[2])



# -*- coding: utf-8 -*-

import codecs
import sys


def sort_items(input_filename, output_filename):
    input_file = codecs.open(input_filename, "r", encoding="utf-8")
    dic = {}

    for line in input_file:
        if len(line) == 0 or line == '\n':
            continue
        try:
            key, value = line.split("\t")
        except ValueError:
            print(line)
        while value[-1] == "\n" or value[-1] == "\r":
            value = value[:-1]
        dic[key] = value

    input_file.close()

    output_file = open(output_filename, "wb")

    for key in sorted(dic.keys()):
        line = key + "\t" + dic[key] + "\n"
        output_file.write(line.encode('utf-8'))

    output_file.close()


def reverse_items(input_filename, output_filename):
    input_file = codecs.open(input_filename, "r", encoding="utf-8")
    dic = {}

    for line in input_file:
        if len(line) == 0:
            continue
        key, value = line.split("\t")
        while value[-1] == "\n" or value[-1] == "\r":
            value = value[:-1]

        value_list = value.split(" ")
        for value in value_list:
            if value in dic:
                dic[value].append(key)
            else:
                dic[value] = [key]

    input_file.close()

    output_file = open(output_filename, "wb")

    for key in sorted(dic.keys()):
        line = key + "\t" + " ".join(dic[key]) + "\n"
        output_file.write(line.encode('utf-8'))

    output_file.close()


def find_target_items(input_filename, keyword):
    input_file = codecs.open(input_filename, "r", encoding="utf-8")
    for line in input_file:
        if len(line) == 0:
            continue
        key, value = line.split("\t")
        while value[-1] == "\n" or value[-1] == "\r":
            value = value[:-1]

        value_list = value.split(" ")
        for value in value_list:
            if keyword in value:
                sys.stdout.write(line)

    input_file.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import sys

from common import sort_items

if len(sys.argv) < 4:
    print("Merge and sort all text dictionaries")
    print(("Usage: ", sys.argv[0], "[input1] [input2] ... [inputN] [output]"))
    exit(1)

all_lines = []
for i in range(1, len(sys.argv) - 1):
    input_file = codecs.open(sys.argv[i], "r", encoding="utf-8")
    for line in input_file:
        all_lines += line
    input_file.close()
    all_lines += '\n'

output_filename = sys.argv[-1]
output_file = open(output_filename, "wb")
for line in all_lines:
    output_file.write(line.encode('utf-8'))
output_file.close()

sort_items(output_filename, output_filename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

from common import reverse_items

if len(sys.argv) != 3:
    print("Reverse key and value of all pairs")
    print(("Usage: ", sys.argv[0], "[input] [output]"))
    exit(1)

reverse_items(sys.argv[1], sys.argv[2])



import os
import re
import subprocess
import sys
import warnings

import setuptools
import setuptools.command.build_ext
import wheel.bdist_wheel

_this_dir = os.path.dirname(os.path.abspath(__file__))
_clib_dir = os.path.join(_this_dir, 'python', 'opencc', 'clib')
_build_dir = os.path.join(_this_dir, 'build', 'python')

_cmake_file = os.path.join(_this_dir, 'CMakeLists.txt')
_author_file = os.path.join(_this_dir, 'AUTHORS')
_readme_file = os.path.join(_this_dir, 'README.md')

try:
    sys.path.insert(0, os.path.join(_this_dir, 'python'))

    import opencc  # noqa
    _libopencc_built = True
except ImportError:
    _libopencc_built = False


def get_version_info():
    version_info = ['1', '0', '0']
    version_pattern = re.compile(
        r'OPENCC_VERSION_(MAJOR|MINOR|REVISION) (\d+)')
    with open(_cmake_file, 'rb') as f:
        for l in f:
            match = version_pattern.search(l.decode('utf-8'))
            if not match:
                continue
            if match.group(1) == 'MAJOR':
                version_info[0] = match.group(2)
            elif match.group(1) == 'MINOR':
                version_info[1] = match.group(2)
            elif match.group(1) == 'REVISION':
                version_info[2] = match.group(2)
    version = '.'.join(version_info)
    return version


def get_author_info():
    if not os.path.isfile(_author_file):
        return 'BYVoid', 'byvoid@byvoid.com'

    authors = []
    emails = []
    author_pattern = re.compile(r'(.+) <(.+)>')
    with open(_author_file, 'rb') as f:
        for line in f:
            match = author_pattern.search(line.decode('utf-8'))
            if not match:
                continue
            authors.append(match.group(1))
            emails.append(match.group(2))

    if len(authors) == 0:
        return 'BYVoid', 'byvoid@byvoid.com'

    return ', '.join(authors), ', '.join(emails)


def get_long_description():
    with open(_readme_file, 'rb') as f:
        return f.read().decode('utf-8')


def build_libopencc():
    if _libopencc_built:
        return  # Skip building binary file
    print('building libopencc into %s' % _build_dir)

    is_windows = sys.platform == 'win32'

    # Make build directories
    if is_windows:
        subprocess.call('md {}'.format(_build_dir), shell=True)
        subprocess.call('md {}'.format(_clib_dir), shell=True)
    else:
        subprocess.call('mkdir -p {}'.format(_build_dir), shell=True)
        subprocess.call('mkdir -p {}'.format(_clib_dir), shell=True)

    # Configure
    cmake_args = [
        '-DBUILD_DOCUMENTATION:BOOL=OFF',
        '-DBUILD_SHARED_LIBS:BOOL=OFF',
        '-DENABLE_GTEST:BOOL=OFF',
        '-DENABLE_BENCHMARK:BOOL=OFF',
        '-DBUILD_PYTHON:BOOL=ON',
        '-DCMAKE_BUILD_TYPE=Release',
        '-DCMAKE_INSTALL_PREFIX={}'.format(_clib_dir),
        '-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={}'.format(_clib_dir),
        '-DPYTHON_EXECUTABLE={}'.format(sys.executable),
    ]

    if is_windows:
        cmake_args += \
            ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_RELEASE={}'.format(_clib_dir)]
        if sys.maxsize > 2**32:
            cmake_args += ['-A', 'x64']

    cmd = ['cmake', '-B', _build_dir] + cmake_args
    errno = subprocess.call(cmd)
    assert errno == 0, 'Configure failed'

    # Build
    cmd = [
        'cmake', '--build', _build_dir,
        '--config', 'Release',
        '--target', 'install'
    ]
    errno = subprocess.call(cmd)
    assert errno == 0, 'Build failed'

    # Empty __init__.py file has to be created
    # to make opencc.clib a module
    with open('{}/__init__.py'.format(_clib_dir), 'w'):
        pass


class OpenCCExtension(setuptools.Extension, object):
    def __init__(self, name, sourcedir=''):
        setuptools.Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class BuildExtCommand(setuptools.command.build_ext.build_ext, object):
    def build_extension(self, ext):
        if isinstance(ext, OpenCCExtension):
            build_libopencc()
        else:
            super(BuildExtCommand, self).build_extension(ext)


class BDistWheelCommand(wheel.bdist_wheel.bdist_wheel, object):
    """Custom bdsit_wheel command that will change
    default plat-name based on PEP 425 and PEP 513
    """

    @staticmethod
    def _determine_platform_tag():
        if sys.platform == 'win32':
            if 'amd64' in sys.version.lower():
                return 'win-amd64'
            return sys.platform

        if sys.platform == 'darwin':
            _, _, _, _, machine = os.uname()
            if machine == 'x86_64':
                return 'macosx-10.9-{}'.format(machine)
            if machine == 'arm64':
                return 'macosx-11.0-{}'.format(machine)
            else:
                raise NotImplementedError
                
        if os.name == 'posix':
            _, _, _, _, machine = os.uname()
            return 'manylinux1-{}'.format(machine)

        warnings.warn(
            'Windows macos and linux are all not detected, '
            'Proper distribution name cannot be determined.')
        from distutils.util import get_platform
        return get_platform()

    def initialize_options(self):
        super(BDistWheelCommand, self).initialize_options()
        self.plat_name = self._determine_platform_tag()


packages = ['opencc', 'opencc.clib']

version_info = get_version_info()
author_info = get_author_info()

setuptools.setup(
    name='OpenCC',
    version=version_info,
    author=author_info[0],
    author_email=author_info[1],
    description=" Conversion between Traditional and Simplified Chinese",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/BYVoid/OpenCC",

    packages=packages,
    package_dir={'opencc': 'python/opencc'},
    package_data={str('opencc'): [
        'clib/opencc_clib*',
        'clib/share/opencc/*',
    ]},
    ext_modules=[OpenCCExtension('opencc.clib.opencc_clib', 'python')],
    cmdclass={
        'build_ext': BuildExtCommand,
        'bdist_wheel': BDistWheelCommand
    },

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: Chinese (Simplified)',
        'Natural Language :: Chinese (Traditional)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Localization',
        'Topic :: Text Processing :: Linguistic',
    ],
    license='Apache License 2.0',
    keywords=['opencc', 'convert', 'chinese']
)



from __future__ import absolute_import, unicode_literals

import os
import sys

from opencc.clib import opencc_clib

__all__ = ['OpenCC', 'CONFIGS', '__version__']

__version__ = opencc_clib.__version__
_thisdir = os.path.dirname(os.path.abspath(__file__))
_opencc_share_dir = os.path.join(_thisdir, 'clib', 'share', 'opencc')

if sys.version_info.major == 2:
    text_type = unicode  # noqa
else:
    text_type = str

if os.path.isdir(_opencc_share_dir):
    CONFIGS = [f for f in os.listdir(_opencc_share_dir) if f.endswith('.json')]
else:
    CONFIGS = []


def _append_path_to_env(name, path):
    value = os.environ.get(name, '')
    if path in value:  # Path already exists
        return
    if value == '':
        value = path
    else:
        value += ':' + path
    os.environ[name] = value


class OpenCC(opencc_clib._OpenCC):

    def __init__(self, config='t2s'):
        if not config.endswith('.json'):
            config += '.json'
        if not os.path.isfile(config):
            config = os.path.join(_opencc_share_dir, config)
        super(OpenCC, self).__init__(config)
        self.config = config

    def convert(self, text):
        if isinstance(text, text_type):
            text = text.encode('utf-8')
        return super(OpenCC, self).convert(text, len(text))
